#include <stdio.h>
#include <stdlib.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "types.h"
#include "ftd2xx.h"

/* macro defines */

#define FTDI_PACKET_SIZE    0xF7D8

/* external variables */

/* None */

/* local variables */

static char buffer[64]; /* buffer to hold device serial number                          */
static char returnbyte = 0;

/* global variables */

FT_HANDLE fthandle;     /* Handle of the device to be opened and used for all functions */
FT_STATUS status;       /* Variable needed for FTDI Library functions                   */
DWORD TxSent;
DWORD RxSent;

/* local prototypes */

/* None */

/*
================================================================
=
= Function: gecko_open
=
= Description:
=
= this function tries to get a handle to the ftdi device
=
================================================================ 
*/
S32 gecko_open (void)
{
    /* Get first device serial number */
    status = FT_ListDevices(0, buffer, FT_LIST_BY_INDEX | FT_OPEN_BY_SERIAL_NUMBER);
	if(status != FT_OK) 
	{
		printf("Couldn't get FTDI device name");
		exit(0);
	}

    /* Open by Serial Number */
    status = FT_OpenEx(buffer, FT_OPEN_BY_SERIAL_NUMBER, &fthandle);
    if(status != FT_OK)
    {
        printf("Error: Couldn't connect to USB Gecko. Please check Installation\n");
        exit(0);
    }

    /* Reset device */                
    status = FT_ResetDevice(fthandle);
    if(status != FT_OK)
    {
        printf("Error: Couldnt Reset Device %d\n",status);
        FT_Close(fthandle);
        exit(0);
    }

    /* no timeout (block) on write and reads */
    status = FT_SetTimeouts(fthandle,0,0);  
    if(status != FT_OK)
    {
        printf("Error: Timeouts failed to set %d\n",status);
        FT_Close(fthandle);
        exit(0);
    }      
    
    /* Purge buffers */                
    status = FT_Purge(fthandle,FT_PURGE_RX);
    if(status != FT_OK)
    {
        printf("Error: Problem clearing buffers %d\n",status);
        FT_Close(fthandle);
        exit(0);
    }

    status = FT_Purge(fthandle,FT_PURGE_TX);
    if(status != FT_OK)
    {
        printf("Error: Problem clearing buffers %d\n",status);
        FT_Close(fthandle);
        exit(0);
    }
    
    /* Set to 64K packet size (USB 2.0 Max) */
    status = FT_SetUSBParameters(fthandle,65536,0); 
    if(status != FT_OK)
    {
        printf("Error: Couldnt Set USB Parameters %d\n",status);
        FT_Close(fthandle);
        exit(0);
    }
    
    return 0;
}

/*
================================================================
=
= Function: gecko_read_byte
=
= Description:
=
= read one byte from the ftdi device
=
================================================================ 
*/
U8 gecko_read_byte(void)
{
    gecko_read (&returnbyte, 1);
    return returnbyte;
}

/*
================================================================
=
= Function: gecko_send_byte
=
= Description:
=
= write one control byte to the ftdi device.
=
================================================================ 
*/
void gecko_send_byte(U8 *byte)
{
    gecko_write(byte, 1);    
}

/*
================================================================
=
= Function: gecko_read
=
= Description:
=
= read count bytes from the ftdi device
=
================================================================ 
*/
S32 gecko_read (void *buf, size_t count)
{
    size_t left;
    size_t chunk;
    left = count;
    
    while(left)
    {
        chunk = left;
        
        if (chunk > FTDI_PACKET_SIZE)
        {
            chunk = FTDI_PACKET_SIZE;
        }

        /* Read in the data */
        status = FT_Read(fthandle, buf, chunk, &RxSent);        
       
        /* Check read ok */
        if (status != FT_OK)
        { 
            printf("Error: Read Error. Closing\n");
            FT_Close(fthandle);    /* Close device as fatal error */
            exit(-1);
        }

        /* bytes read ok, read next chunk */
        left -= chunk;
        buf += chunk;
    }

    return 0;
}

/*
================================================================
=
= Function: gecko_write
=
= Description:
=
= write count bytes to the ftdi device 
=
================================================================ 
*/
S32 gecko_write(void *buf, size_t count)
{
    size_t left;
    size_t chunk;
    left = count;

    while(left)
    {
        chunk = left;
        
        if (chunk > FTDI_PACKET_SIZE)
        {
            chunk = FTDI_PACKET_SIZE;
        }  

        /* write data */
        status = FT_Write(fthandle, buf, chunk, &TxSent);       

        /* check write ok */
        if (status != FT_OK)
        {  
            printf("Error: Write Error. Closing.\n");
            FT_Close(fthandle);    /* Close device as fatal error */
            exit(-1);
        }
        
        left -= chunk;
        buf += chunk;
    }   

    return 0;
}

/*
================================================================
=
= Function: gecko_close
=
= Description:
=
= close the handle to the ftdi device
=
================================================================ 
*/
void gecko_close(void)
{
    printf("\nplease unplug USB cable\n");
    FT_Close(fthandle);
}
